@@ -161,6 +161,7 @@ class DSI():
             'AORprintF': False,  # Print bisection output
             
             # ----- Hull Parameters ----- #
+            'no_splits': 10, # Splits for the inside3D function to reduce memory req
             'a': None,        # Alpha value -> at large alpha,
                               # hull becomes convex. if None: use product of bounds range
             'amul': 1,        # Alpha multiplier value (wrt to a used)
@@ -432,9 +433,6 @@ class DSI():
         else:
             self.vnames = vnames
         points = sat[vnames].to_numpy(dtype='float')
-        if len(extra_points) != 0:
-            points = np.vstack([points, extra_points])
-            self.extra_points = extra_points
         vio = self.vio
         vpoints = vio[vnames].to_numpy(dtype='float')
         self.opt.update({'vnames': vnames})
@@ -442,6 +440,9 @@ class DSI():
         print_flag = self.opt['printF']
         opt = self.opt
         extra_points = opt['extra_points']
+        if len(extra_points) != 0:
+            points = np.vstack([points, extra_points])
+            self.extra_points = extra_points
         a = opt['a']
         if a == None:
             fdf = pd.concat([sat, vio], axis = 0)[vnames]
@@ -462,10 +463,10 @@ class DSI():
             shp = find_shp(points, alpha)
             shp['amul'] = amul
             sol_flag = f'No optimisation performed'
-            opt_log = 'N/A'
+            # opt_log = 'N/A'
 
         elif opt['opt_amul'] == True: # Use bisection
-            opt_log = []
+            # opt_log = []
             tol = opt['tol']
             lb = opt['lb']
             ub = opt['ub']
@@ -476,7 +477,7 @@ class DSI():
                 shp = find_shp(points, a*mp)
                 shp['amul'] = mp
                 shp['iter'] = i + 1
-                opt_log.append(shp)
+                # opt_log.append(shp)
                 self.shp = shp
                 r = inside(vpoints, shp)
                 flag = True in r
@@ -503,6 +504,12 @@ class DSI():
         if print_flag:
             print(sol_flag)
 
+        # Calculate volume
+        if dim == 3:
+            volume = 0
+            for v in shp['P'][shp['tetras']]:
+                volume += np.abs(np.dot(v[0] - v[3], np.cross(v[1] - v[3], v[2] - v[3])))/6
+            shp['size'] = volume
         self.vindsp = vio[inside(vpoints, shp)]
         self.indsp = pd.concat([sat, self.vindsp]).reset_index()
         space_size = shp['size']
@@ -512,7 +519,7 @@ class DSI():
         self.P = shp['P']
         self.report['space_size'] = space_size
         self.report['sol_flag'] = sol_flag
-        self.report['opt_log'] = opt_log
+        # self.report['opt_log'] = opt_log
         return shp
     
     def alphashape_2D(self, P, alpha):
@@ -600,7 +607,7 @@ class DSI():
         """
         from scipy.spatial import Delaunay
         import numpy as np
-        from collections import defaultdict
+        import pandas as pd
         tetra = self.tetra
         r = self.r
         if tetra == None:
@@ -632,10 +639,7 @@ class DSI():
         triangles = np.sort(triangles, axis = 1)
 
         # Remove triangles that occurs twice, because they are within shapes
-        trianglesDict = defaultdict(int)
-        for tri in triangles:
-            trianglesDict[tuple(tri)] += 1
-        triangles = np.array([tri for tri in trianglesDict if trianglesDict[tri] == 1])
+        triangles = pd.DataFrame(triangles).drop_duplicates(keep = False).to_numpy()
 
         #edges
         edgeComb = np.array([(0, 1), (0, 2), (1, 2)])
@@ -654,10 +658,10 @@ class DSI():
         shp['edges_val'] = None
         shp['alpha'] = alpha
 
-        volume = 0
-        for v in P[tetras]:
-            volume += np.abs(np.dot(v[0] - v[3], np.cross(v[1] - v[3], v[2] - v[3])))/6
-        shp['size'] = volume
+        # volume = 0
+        # for v in P[tetras]:
+        #     volume += np.abs(np.dot(v[0] - v[3], np.cross(v[1] - v[3], v[2] - v[3])))/6
+        shp['size'] = 420e-10
         return shp
 
     def collect_frames(self, anielev = 10, sfolder = 'animation', sname = 'plot',\
@@ -721,31 +725,45 @@ class DSI():
             shp = self.shp
         dim = len(np.array(x).shape)
 
+        no_x = np.array(x).shape[0]
+        if dim == 2:
+            no_splits = self.opt['no_splits']
+            if no_x < no_splits:
+                no_splits = int(no_x/2)
+            x_list = np.array_split(x, no_splits)
+        else:
+            x_list = [x]
+
         # Find which tetrahedron the point lies in
         node_coordinates = shp['P']
         node_ids = shp['tetras']
-        p = np.array(x)
 
-        ori = node_coordinates[node_ids[:, 0],:]
-        v1 = node_coordinates[node_ids[:, 1],:] - ori
-        v2 = node_coordinates[node_ids[:, 2],:] - ori
-        v3 = node_coordinates[node_ids[:, 3],:] - ori
-        n_tet = len(node_ids)
-        v1r = v1.T.reshape((3, 1, n_tet))
-        v2r = v2.T.reshape((3, 1, n_tet))
-        v3r = v3.T.reshape((3, 1, n_tet))
-        mat = np.concatenate((v1r, v2r, v3r), axis=1)
-        inv_mat = np.linalg.inv(mat.T).T
-        if p.size == 3:
-            p = p.reshape((1,3))
-        n_p = p.shape[0]
-        orir = np.repeat(ori[:,:,np.newaxis], n_p, axis=2)
-        newp = np.einsum('imk,kmj->kij',inv_mat,p.T-orir)
-        val = np.all(newp>=0, axis=1) & np.all(newp <=1, axis=1) & (np.sum(newp, axis=1)<=1)
-        id_tet, id_p = np.nonzero(val)
-        res = -np.ones(n_p, dtype=id_tet.dtype) # Sentinel value
-        res[id_p]=id_tet
-        
+        res_list = []
+        for x_s in x_list:
+            p = np.array(x_s)
+
+            ori = node_coordinates[node_ids[:, 0],:]
+            v1 = node_coordinates[node_ids[:, 1],:] - ori
+            v2 = node_coordinates[node_ids[:, 2],:] - ori
+            v3 = node_coordinates[node_ids[:, 3],:] - ori
+            n_tet = len(node_ids)
+            v1r = v1.T.reshape((3, 1, n_tet))
+            v2r = v2.T.reshape((3, 1, n_tet))
+            v3r = v3.T.reshape((3, 1, n_tet))
+            mat = np.concatenate((v1r, v2r, v3r), axis=1)
+            inv_mat = np.linalg.inv(mat.T).T
+            if p.size == 3:
+                p = p.reshape((1,3))
+            n_p = p.shape[0]
+            orir = np.repeat(ori[:,:,np.newaxis], n_p, axis=2)
+            newp = np.einsum('imk,kmj->kij',inv_mat,p.T-orir)
+            val = np.all(newp>=0, axis=1) & np.all(newp <=1, axis=1) & (np.sum(newp, axis=1)<=1)
+            id_tet, id_p = np.nonzero(val)
+            res = -np.ones(n_p, dtype=id_tet.dtype) # Sentinel value
+            res[id_p] = id_tet
+            res_list.append(res)
+        res = np.concatenate(res_list)
+
         if dim == 1:
             x = [x]
         # return res
